# hglm estimates for one simulation replicate
###################################################
# SET SIMULATION PARAMETERS
###################################################
rm(list=ls())
library(hglm)
source("File S2.R")
set.seed(9998)

#Set values for the simulation
# Values for population

n.snp.per.chrom=1000

n.ind.per.gen=200
Ne=100
M=100
n.gen=400

# Values for genome
n.chrom=2
n.qtl.per.chrom=10
n.qtl=n.chrom*n.qtl.per.chrom



n.markers.per.chrom=n.snp.per.chrom+n.qtl.per.chrom
multipl=4
n.loci.per.chrom.sim=round(n.markers.per.chrom*multipl)

# Values for trait
scenario=0
step.size.cat=c(10,20,40)
step.size.con=1
cases=(2*length(step.size.cat))+length(step.size.con)
fixed=0
hsq=0.2
varTBVfixed=1
qtlMAF=0
beta.lower=1
beta.upper=3


#------------------------------------------------------------------------------------------------------------------------------
# SIMULATION 

# START THE SIMULATION
	# Simulate the population
	enough.non.fixed.markers=F
	while(enough.non.fixed.markers==F) {
		Zsim=Simulate.Genotypes.2generations(n.chrom=n.chrom, n.markers=n.loci.per.chrom.sim, n.ind.f=n.ind.per.gen, M=M, Ne=Ne, n.gen=n.gen)
		Ztrain=Zsim[[1]]
		Ztest=Zsim[[2]]
		freq.train=Zsim[[3]]
		freq.new=Zsim[[4]]
		rm(Zsim)

		# Remove all fixed markers from generation 1, and remove the same markers from generation 2
		# Keep track of the positions the markers had before removing the fixed
		Zsim.remove.fixed= Remove.Fixed.Markers(freq.train,keep.old.positions=T)
		old.pos1=Zsim.remove.fixed[[3]]
		old.pos2=Zsim.remove.fixed[[4]]
		segregate1=Zsim.remove.fixed[[1]]
		segregate2=Zsim.remove.fixed[[2]]
		rm(Zsim.remove.fixed)

		# Chech if we have enough segregating markers
		if (segregate1 >= n.markers.per.chrom && segregate2 >= n.markers.per.chrom) {
			enough.non.fixed.markers=T } else {n.loci.per.chrom.sim=round(n.loci.per.chrom.sim*1.2)}
		}
	# Choose SNPs and QTLs for the final populations
	Zsim.final = Draw.QTL.and.SNPs(Ztrain,Ztest,freq.train,freq.new,n.markers.per.chrom,n.qtl.per.chrom,segregate1,segregate2,old.pos1,old.pos2,qtlMAF)
	Ztrain.snp = Zsim.final[[1]] # Consists now of correct amount of markers and possible QTL on each chromosome
	Ztrain.qtl = Zsim.final[[2]]
	Ztest.snp = Zsim.final[[3]] # Consists now of correct amount of markers and possible QTL on each chromosome
	Ztest.qtl = Zsim.final[[4]]
	snp.pos.c1 = Zsim.final[[5]] # Has the posistion of all snp on chromosome 1
	snp.pos.c2 = Zsim.final[[6]] # Has the posistion of all snp on chromosome 2
	qtl.pos.c1 = Zsim.final[[7]] # Has the posistion of all qtl on chromosome 1
	qtl.pos.c2 = Zsim.final[[8]] # Has the posistion of all qtl on chromosome 2
	freq.train.qtl=Zsim.final[[9]]
	freq.train.snp=Zsim.final[[10]]
	freq.test.qtl=Zsim.final[[11]]
	freq.test.snp=Zsim.final[[12]]
	rm(Zsim.final)
	rm(Ztrain)
	rm(Ztest)		

	# Assign variances to the QTL effects depending on the set scenario
	variances=array(,dim=c(2,n.qtl.per.chrom))
	if(scenario==0) {variances[1,]=rep(exp(beta.lower),n.qtl.per.chrom)
				variances[2,]=rep(exp(beta.lower),n.qtl.per.chrom)}
	if(scenario!=0) stop("Only Scenario 0 possible in this example code")

	# Create QTL effects, TBVs, Phenotypes
	Trait=Create.trait(Ztrain=Ztrain.qtl,Ztest=Ztest.qtl,varTBVfixed=varTBVfixed,n.ind=n.ind.per.gen,n.qtl=n.qtl.per.chrom,here=hsq,fixed=fixed,var=variances)	
	breed.true=Trait[[1]]
	breed.true.new=Trait[[2]]
	pheno.true=Trait[[3]]
	resid.var=Trait[[4]]
	u=Trait[[5]]
	u.scaled=Trait[[6]]
	breed.true.inscaled=Trait[[7]]
	rm(Trait)

	# Create Zscaled matrixes for evaluation
	New.Z=Scale.Z(Ztrain.snp,Ztest.snp,freq.train.snp,n.ind.per.gen,n.snp.per.chrom)		
	Ztrain.sc=New.Z[[1]]
	Ztest.sc=New.Z[[2]]
	Ztrain.temp=New.Z[[3]]
	Ztest.temp=New.Z[[4]]
	rm(New.Z)
#--------------------------------------------------------------------------------------------------------------------------------------------------
	########################################################################
	# Start Evaluations for alternative models
	#************************************************************************
	SIMULATED.EBV <- cbind(breed.true, breed.true.new)
	SIMULATED.QTL <- rbind(u, qtl.pos.c1, qtl.pos.c2)
	SIMULATED.SNP <- rbind(snp.pos.c1, snp.pos.c2 )
	RESULTS <- list(SNP.effects = NULL, EBVtrain =NULL, EBVtest = NULL, ModelName = NULL, WindowSize = NULL, Converged = NULL)
	LD.models <- c("SNPblup", "Categorical", "Continuous", "LD-window") 
	
	kk = 0
	for (i.model in 1:length(LD.models)) {
	  if (LD.models[i.model] ==  "SNPblup") n.window.sizes = 1
	  #*********************************************************************************************************
	  # W10, W20 and W40 models
	  # Create D.list first
	  if (LD.models[i.model] ==  "Categorical") {
	    D.list_cat=D.list.simple.cat(n.snp.per.chrom,step.size.cat,qtl.pos.c1,qtl.pos.c2,snp.pos.c1,snp.pos.c2)
	    D.list=D.list_cat[[1]]
	    n.window.sizes = length(step.size.cat)
	  }
	  #*********************************************************************************************************
	  # LD continuous model
	  # Create D.list first
	  if (LD.models[i.model] ==  "Continuous") {
	    D.list_con=D.list.LD.con(n.ind.per.gen,n.snp.per.chrom,n.qtl.per.chrom,Ztrain.snp,Ztrain.qtl,freq.train.snp,freq.train.qtl)
	    D.list=D.list_con[[1]]
	    n.window.sizes = 1
	  }
	  #*********************************************************************************************************
	  # W10, W20, W40 models
	  # Create D.list first
	  if (LD.models[i.model] ==  "LD-window") {
	    D.list_LDcat=D.list.LD.cat(n.ind.per.gen,n.snp.per.chrom,step.size.cat,Ztrain.snp,Ztrain.qtl,qtl.pos.c1,qtl.pos.c2,snp.pos.c1,snp.pos.c2,freq.train.snp,freq.train.qtl)
	    D.list=D.list_LDcat[[1]]
	    n.window.sizes = length(step.size.cat)
	  }
	  #*********************************************************************************************************
	   for (i in 1:n.window.sizes) {
	    kk = kk +1
	    if (LD.models[i.model] == "SNPblup") {X.rand.D = NULL} else {X.rand.D = list(model.matrix(~D.list[[i]]))}
	    mod = try(hglm(y = pheno.true, X = matrix(1, n.ind.per.gen, 1), Z = Ztrain.sc, conv = 1e-6,X.rand.disp = X.rand.D))
	    if(class(mod)=="try-error" || mod$iter<=3) {
	      mod = try(hglm(y = pheno.true, X = matrix(1, n.ind.per.gen, 1), Z = Ztrain.sc, conv = 1e-8,X.rand.disp = X.rand.D))
	    }
		if(class(mod)!="try-error") {
	    RESULTS$SNP.effects[[kk]] = mod$ranef
	    # Predict on same generation
	    RESULTS$EBVtrain[[kk]]  = Ztrain.sc%*%mod$ranef
	    # Predict on generation 2
	    RESULTS$EBVtest[[kk]] =  Ztest.sc%*%mod$ranef
	    RESULTS$ModelName[[kk]] = LD.models[i.model]
	    tmp = 1
	    if (LD.models[i.model] == "Categorical") tmp = step.size.cat[i]
	    RESULTS$WindowSize[[kk]] = tmp
	    RESULTS$Converged[[kk]] = mod$Converge
		}
		else {
	    RESULTS$Converged[[kk]] = 0 }
	  }
	}

	titles = c("SNP-BLUP", "W10", "W20", "W40", "LD", "W10-LD", "W20-LD", "W40-LD" )
	
	if (sum(RESULTS$Converged == "converged")  != 0) {
	  cat("The following models converged: \n")
	  cat(titles[RESULTS$Converged == "converged"], "\n")
	}
	
	if (sum(RESULTS$Converged != "converged")  != 0) {
	  cat("The following models DID NOT converge: \n")
	  cat(titles[RESULTS$Converged != "converged"], "\n")
	}
	
	# Plot SNP-effects
	n.models = kk
	
	maxY = max(unlist(lapply(RESULTS$SNP.effects,max)))+0.4
	colors = as.matrix(c("firebrick2","olivedrab3","olivedrab2","olivedrab1","darkorchid2","deepskyblue3","deepskyblue2","deepskyblue1"))

	pdf(file = paste0("SnpEffectsFigure", n.snp.per.chrom ,".pdf" ),width=6,height=9)
	#tiff(file = paste0("SnpEffectsFigure", n.snp.per.chrom ,".tiff" ),height=7.5,width=5,units='in',res=300)
	par(mfcol=c(4,2))
	for(k in 1:n.models) {
	  if (RESULTS$Converged[[k]]=="converged") {
	  plot(c(snp.pos.c1, snp.pos.c2+n.loci.per.chrom.sim),abs( RESULTS$SNP.effects[[k]]),type="h", 
	       col = colors[k],xlim = c(0, n.chrom*n.loci.per.chrom.sim), ylim=c(0, maxY),lwd=1,
	       xlab="Position", ylab="Absoulte effect", main=titles[k],cex.main=1.1,cex.axis=1.1,cex.lab=1.1)
	  points(c(qtl.pos.c1, qtl.pos.c2+n.loci.per.chrom.sim), abs(as.numeric(t(u.scaled))), pch=19, cex=0.5) 
	  }
	  
	}
	dev.off()
	

	