Simulate.Genotypes.2generations = function(n.chrom, n.markers, n.ind.f, M, Ne, n.gen, filename="test", lambda.rec=1, lambda.mut=1) {
  # Function that simulates two generations of a population. In the historical generations we have both mutation and recombination. In generation 1 and 2 we only have recombination.
  #
  # INPUT
  # n.chrom: Nr of chromosomes
  # n.markers: Number of markers per chromosome
  # n.ind.f: Nr of individuals per generation in final population
  # Ne: Nr of individuals per generation in historical generations
  # M: Nr of individuals in the last historical generation
  # n.gen: Nr of historical generations minus 1
  # lambda.rec: Parameter in Poisson distribution. Nr of recombinations per chromosome is Poission distributed
  # lambda.mut: Parameter in Poisson distribution. Nr of mutations per chromosome is Poission distributed
  #
  # OUTPUT
  # Z: Genotype matrix for generation 1. Markers have genotypes 0, 1 and 2
  # Znew:  Genotype matrix for generation 2. Markers have genotypes 0, 1 and 2
  # freq.train: MAF for generaion 1
  # freq.test: MAF for generation 2
  
  # Assign alleles to all individuals in pop1 randomly
  tot.allele = Ne*n.chrom*n.markers*2
  pop1 = array(round(runif(tot.allele, min = 1, max = 2)),dim = c(Ne,n.chrom, n.markers, 2))
  # Create the historical populations
  pop2 = array(NA,dim = c(Ne,n.chrom, n.markers, 2)) # Helper array
  for(m in 1:n.gen){
    # Draw mothers and fathers
    mothers.mate = sample(size = Ne, x = c(1:Ne), replace = T) # The mothers of new individuals
    # Make sure not get same parent twice
    prob.f = numeric(Ne)+ 1
    prob.f[mothers.mate] = 0
    fathers.mate = sample(size = Ne, x = c(1:Ne), replace = T, prob = prob.f) # the fathers of new individuals
    for (i in 1:Ne){ # For each new individual
      iale.p = sample(size = n.chrom, c(1,2), replace = T) # Which allele from father will be inherited
      iale.m = sample(size = n.chrom, c(1,2), replace = T) # Which allele from mother will be inherited
      n.rec.p = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from father
      n.rec.m = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from mother
      for(j in 1:n.chrom){ # For each chromosome
        # Male
        # Recombination
        if(n.rec.p[j]==0){ # If there is no recombination, the offspring recieves the fathers chromosome chosen for breeding
          pop2[i,j,,1]=pop1[fathers.mate[i],j,,iale.p[j]]
        }
        else{
          recom.markers = sort(sample(size = n.rec.p[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
          pop2[i,j,,1]=pop1[fathers.mate[i],j,,iale.p[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
          for (s in 1:length(recom.markers)){
            if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
              pop2[i,j,(recom.markers[s] : n.markers),1] = pop1[fathers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.p[j]-3)]
            }
            else{
              pop2[i,j,(recom.markers[s] : n.markers),1] = pop1[fathers.mate[i],j,(recom.markers[s] : n.markers),iale.p[j]]
            }
          }
        }
        # Mutation
        n.mut = rpois(n = 1, lambda = lambda.mut)
        mut.spots = sample(size = n.mut, c(1:n.markers))
        pop2[i,j,mut.spots,1] = abs(pop2[i,j,mut.spots,1] -3)
        
        # Female
        # Recombination
        if(n.rec.m[j]==0){
          pop2[i,j,,2]=pop1[mothers.mate[i],j,,iale.m[j]]
        }
        else{
          recom.markers = sort(sample(size = n.rec.m[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
          pop2[i,j,,2]=pop1[mothers.mate[i],j,,iale.m[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
          for (s in 1:length(recom.markers)){
            if(s%%2 !=0){
              pop2[i,j,(recom.markers[s]:n.markers),2] = pop1[mothers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.m[j]-3)]
            }
            else{
              pop2[i,j,(recom.markers[s]:n.markers),2] = pop1[mothers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.m[j])]
            }
          }
        }
        # Mutation
        n.mut = rpois(n = 1, lambda = lambda.mut)
        mut.spots = sample(size = n.mut, c(1:n.markers))
        pop2[i,j,mut.spots,2] = abs(pop2[i,j,mut.spots,2] -3)
      }
    }
    pop1 = pop2
  }
  # Now pop2 is the last historical population
  
  # Expanding the population to m individuals
  # Draw mothers and fathers
  mothers.mate = sample(size = M, x = c(1:Ne), replace = T) # the mothers of new individuals
  prob.f = numeric(Ne)+ 1
  prob.f[mothers.mate] = 0
  fathers.mate = sample(size = M, x = c(1:Ne), replace = T, prob = prob.f) # the fathers of new individuals
  pop3 = array(NA,dim = c(M,n.chrom, n.markers, 2))
  for (i in 1:M){ # For each new individual
    iale.p = sample(size = n.chrom, c(1,2), replace = T) # Which allele from father will be inherited
    iale.m = sample(size = n.chrom, c(1,2), replace = T) # Which allele from mother will be inherited
    n.rec.p = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from father
    n.rec.m = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from mother
    if(mothers.mate[i]==fathers.mate[i]){samme =samme+1}
    for(j in 1:n.chrom){ # For each chromosome
      # Male
      # Recombination
      if(n.rec.p[j]==0){
        pop3[i,j,,1]=pop1[fathers.mate[i],j,,iale.p[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.p[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop3[i,j,,1]=pop1[fathers.mate[i],j,,iale.p[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop3[i,j,(recom.markers[s] : n.markers),1] = pop1[fathers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.p[j]-3)]
          }
          else{
            pop3[i,j,(recom.markers[s] : n.markers),1] = pop1[fathers.mate[i],j,(recom.markers[s] : n.markers),iale.p[j]]
          }
        }
      }
      # Female
      # Recombination
      if(n.rec.m[j]==0){
        pop3[i,j,,2]=pop1[mothers.mate[i],j,,iale.m[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.m[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop3[i,j,,2]=pop1[mothers.mate[i],j,,iale.m[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop3[i,j,(recom.markers[s] : n.markers),2] = pop1[mothers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.m[j]-3)]
          }
          else{
            pop3[i,j,(recom.markers[s] : n.markers),2] = pop1[mothers.mate[i],j,(recom.markers[s] : n.markers),iale.m[j]]
          }
        }
      }
    }
  }
  
  # Now pop3 is an expanded population consisting of M individuals
  
  # Final populations
  # The final population will be stored in pop4
  pop4 = array(NA,dim = c(n.ind.f,n.chrom, n.markers, 2))
  mothers.mate = sample(size = n.ind.f, x = c(1:M), replace = T) # the mothers of new individuals
  prob.f = numeric(M)+ 1
  prob.f[mothers.mate] = 0
  fathers.mate = sample(size = n.ind.f, x = c(1:M), replace = T, prob = prob.f) # the fathers of new individuals
  for(i in 1:n.ind.f){
    iale.p = sample(size = n.chrom, c(1,2), replace = T) # Which allele from father will be inherited
    iale.m = sample(size = n.chrom, c(1,2), replace = T) # Which allele from mother will be inherited
    n.rec.p = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from father
    n.rec.m = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from mother
    if(mothers.mate[i]==fathers.mate[i]){samme =samme+1}
    for (j in 1:n.chrom){
      # Male part
      if(n.rec.p[j]==0){ # If there are no recombinations for this chromosome
        pop4[i,j,,1]= pop3[fathers.mate[i], j,,iale.p[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.p[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop4[i,j,,1]=pop3[fathers.mate[i],j,,iale.p[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop4[i,j,(recom.markers[s] : n.markers),1] = pop3[fathers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.p[j]-3)]
          }
          else{
            pop4[i,j,(recom.markers[s] : n.markers),1] = pop3[fathers.mate[i],j,(recom.markers[s] : n.markers),iale.p[j]]
          }
        }
      }
      # Female part
      if(n.rec.m[j]==0){ # If there are no recombinations for this chromosome
        pop4[i,j,,2]= pop3[mothers.mate[i], j,,iale.m[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.m[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop4[i,j,,2]=pop3[mothers.mate[i],j,,iale.m[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop4[i,j,(recom.markers[s] : n.markers),2] = pop3[mothers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.m[j]-3)]
          }
          else{
            pop4[i,j,(recom.markers[s] : n.markers),2] = pop3[mothers.mate[i],j,(recom.markers[s] : n.markers),iale.m[j]]
          }
        }
      }
    }
  }
  
  # Simulate one new generation so that can predict on new generation, and not only on new population
  pop6 = pop4
  pop.new = array(NA,dim = c(n.ind.f,n.chrom, n.markers, 2))
  mothers.mate = sample(size = n.ind.f, x = c(1:M), replace = T) # the mothers of new individuals
  prob.f = numeric(M)+ 1
  prob.f[mothers.mate] = 0
  fathers.mate = sample(size = n.ind.f, x = c(1:M), replace = T, prob = prob.f) # the fathers of new individuals
  for(i in 1:n.ind.f){
    iale.p = sample(size = n.chrom, c(1,2), replace = T) # Which allele from father will be inherited
    iale.m = sample(size = n.chrom, c(1,2), replace = T) # Which allele from mother will be inherited
    n.rec.p = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from father
    n.rec.m = rpois(n = n.chrom, lambda = lambda.rec) # Number of recombinations in chrom from mother
    for (j in 1:n.chrom){
      # Male part
      if(n.rec.p[j]==0){ # If there are no recombinations for this chromosome
        pop.new[i,j,,1]= pop6[fathers.mate[i], j,,iale.p[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.p[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop.new[i,j,,1]=pop6[fathers.mate[i],j,,iale.p[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop.new[i,j,(recom.markers[s] : n.markers),1] = pop6[fathers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.p[j]-3)]
          }
          else{
            pop.new[i,j,(recom.markers[s] : n.markers),1] = pop6[fathers.mate[i],j,(recom.markers[s] : n.markers),iale.p[j]]
          }
        }
      }
      # Female part
      if(n.rec.m[j]==0){ # If there are no recombinations for this chromosome
        pop.new[i,j,,2]= pop6[mothers.mate[i], j,,iale.m[j]]
      }
      else{
        recom.markers = sort(sample(size = n.rec.m[j], x = c(1:n.markers), replace = F)) # Which markers should have recombinations
        pop.new[i,j,,2]=pop6[mothers.mate[i],j,,iale.m[j]] # The offspring recieves the full chromosome first, then we go through the positions with recombination and change all markers after this point
        for (s in 1:length(recom.markers)){
          if(s%%2 !=0){ # The first, third and so on time with recombination it is the allele not chosen for breeding that is ised, on the second, fourth and so on it is the original allele that is switched back
            pop.new[i,j,(recom.markers[s] : n.markers),2] = pop6[mothers.mate[i],j,(recom.markers[s] : n.markers),abs(iale.m[j]-3)]
          }
          else{
            pop.new[i,j,(recom.markers[s] : n.markers),2] = pop6[mothers.mate[i],j,(recom.markers[s] : n.markers),iale.m[j]]
          }
        }
      }
    }
  }
  # What is the minor allele frequency?
  
  # Go through all markers and for each find the minor allele frequency
  freq4 = matrix(,nrow=n.chrom,ncol=n.markers)
  freq.new = matrix(,nrow=n.chrom,ncol=n.markers)
  for (j in 1:n.chrom){
    for (i in 1:n.markers){
      freq4[j,i] =  sum((pop4[,j,i,1]+pop4[,j,i,2])-2)/(2*n.ind.f)
      
      freq.new[j,i] =  sum((pop.new[,j,i,1]+pop.new[,j,i,2])-2)/(2*n.ind.f)
    }
  }
  return( list(Z = pop4, Znew = pop.new, freq.train = freq4, freq.new=freq.new ) )
}
#---------------------------------------------------------------------------------------------------------
# Removes fixed markers and markers with MAF less than 0.05
Remove.Fixed.Markers = function(freq, keep.old.positions=T){
  # Function that removes markers that are fixed in generation 1, and removes the same markers in generation 2. It also keeps track of position of remaining markers
  # INPUT:
  # freq: frequencies in training pop
  # keep.old.positions: T if want to keep the positions of the markers that are not removed
  n.markers=dim(freq)[2]
  # Remove all with MAF less than 0.05
  keep.this1=matrix()
  in1=0
  for (i in 1:n.markers) {
    if (freq.train[1,i]>=0.05 && freq.train[1,i]<=0.95) {
      in1=in1+1
      keep.this1[in1]=i }
  }
  keep.this2=matrix()
  in2=0
  for (i in 1:n.markers) {
    if (freq.train[2,i]>=0.05 && freq.train[2,i]<=0.95) {
      in2=in2+1
      keep.this2[in2]=i }
  }
  return(list(seg1=in1,seg2=in2,old.pos1=keep.this1,old.pos2=keep.this2))
}


#-----------------------------------------------------------------------------------------------------------------------
# Draws markers for the analysis. Can control the portion of markers with MAF less than 0.1 and 0.2 that are chosen if want
Draw.QTL.and.SNPs = function(Ztrain, Ztest, freq, freq.new, n.markers.per.chrom, n.qtl, seg1, seg2, pos1, pos2, MAF ){
  # Function that randomly selects n.markers.per.chrom markers from the two chromosomes in the genotype matrix of generation 1 and selects the same markers from generation 2. It also keeps track of marker positions in the two chromosomes.
  # INPUT:
  # Ztrain: Population matrix for generation 1 
  # Ztest: Population matrix for generation 2 
  # freq: frequencies in generation 1
  # freq.new: frequencies in generation 2
  # n.markers.per.chrom: How many markers we want on each chromsome
  # n.qtl: Number of QTLs per chromosome
  # seg1: number of segregating markers in chromosome 1
  # seg2: number of segregating markers in chromosome 2
  # pos1: Positions of seg markers chromosome 1
  # pos2: Positions of seg markers chromosome 2
  # MAF: frequency threshold for QTLs
  # OUTPUT:
  # snp.pos.c1: Position of snp on chromosome 1
  # snp.pos.c2: Position of snp on chromosome 2
  # qtl.pos.c1: Position of QTL on chromosome 1
  # qtl.pos.c2: Position of QTL on chromosome 2
  # Ztrain.snp: only SNPS
  # Ztest.snp: only SNPS
  # Ztrain.qtl: only QTLs
  # Ztest.qtl: only QTLs
  # freq.train.qtl: 
  # freq.train.snp: 
  # freq.test.qtl: 
  # freq.test.snp: 
  
  prob.select.as.snp1=rep(1,times=segregate1)
  prob.select.as.snp2=rep(1,times=segregate2) 
  chrom1=sort(sample(x=1:segregate1,size=n.markers.per.chrom,replace=F,prob = prob.select.as.snp1))
  select1=old.pos1[chrom1]
  freq1=freq[1,select1]
  chrom2=sort(sample(x=1:segregate2,size=n.markers.per.chrom,replace=F,prob = prob.select.as.snp2))
  select2=old.pos2[chrom2]
  freq2=freq[2,select2]
  
  if (MAF==0) {
    prob.select.as.qtl1=rep(1,times=n.markers.per.chrom)
    prob.select.as.qtl2=rep(1,times=n.markers.per.chrom) 
  }
  if (MAF!=0) {
    prob.select.as.qtl1=numeric(length(n.markers.per.chrom))
    for (i in 1:n.markers.per.chrom) {
      if (freq1[i]<=MAF) {
        prob.select.as.qtl1[i]=1 }
      else {prob.select.as.qtl1[i]=0}
    }
    prob.select.as.qtl2=numeric(length(n.markers.per.chrom))
    for (i in 1:n.markers.per.chrom) {
      if (freq2[i]<=MAF) {
        prob.select.as.qtl2[i]=1 }
      else {prob.select.as.qtl2[i]=0}
    }
  }
  qtl1=sort(sample(x=1:n.markers.per.chrom,size=n.qtl,replace=F,prob = prob.select.as.qtl1))
  qtl.pos1=select1[qtl1]
  snp.pos1=select1[-qtl1]
  
  qtl2=sort(sample(x=1:n.markers.per.chrom,size=n.qtl,replace=F,prob = prob.select.as.qtl2))
  qtl.pos2=select2[qtl2]
  snp.pos2=select2[-qtl2]
  
  Ztrain.snp=array(,dim=c(dim(Ztrain)[1],2,length(snp.pos1),2))
  Ztrain.snp[,1,,]=Ztrain[,1,snp.pos1,]
  Ztrain.snp[,2,,]=Ztrain[,2,snp.pos2,]
  Ztrain.qtl=array(,dim=c(dim(Ztrain)[1],2,length(qtl.pos1),2))
  Ztrain.qtl[,1,,]=Ztrain[,1,qtl.pos1,]
  Ztrain.qtl[,2,,]=Ztrain[,2,qtl.pos2,]
  
  Ztest.snp=array(,dim=c(dim(Ztrain)[1],2,length(snp.pos1),2))
  Ztest.snp[,1,,]=Ztest[,1,snp.pos1,]
  Ztest.snp[,2,,]=Ztest[,2,snp.pos2,]
  Ztest.qtl=array(,dim=c(dim(Ztrain)[1],2,length(qtl.pos1),2))
  Ztest.qtl[,1,,]=Ztest[,1,qtl.pos1,]
  Ztest.qtl[,2,,]=Ztest[,2,qtl.pos2,]
  
  freq.train.qtl=array(,dim=c(2,n.qtl))
  freq.train.qtl[1,]=freq[1,qtl.pos1]
  freq.train.qtl[2,]=freq[2,qtl.pos2]
  freq.train.snp=array(,dim=c(2,n.markers.per.chrom-n.qtl))
  freq.train.snp[1,]=freq[1,snp.pos1]
  freq.train.snp[2,]=freq[2,snp.pos2]
  
  freq.test.qtl=array(,dim=c(2,n.qtl))
  freq.test.qtl[1,]=freq.new[1,qtl.pos1]
  freq.test.qtl[2,]=freq.new[2,qtl.pos2]
  freq.test.snp=array(,dim=c(2,n.markers.per.chrom-n.qtl))
  freq.test.snp[1,]=freq.new[1,snp.pos1]
  freq.test.snp[2,]=freq.new[2,snp.pos2]
  
  return(list(Ztrain.snp=Ztrain.snp,Ztrain.qtl=Ztrain.qtl,Ztest.snp=Ztest.snp,Ztest.qtl=Ztest.qtl,snp.pos.c1=snp.pos1,snp.pos.c2=snp.pos2,qtl.pos.c1=qtl.pos1,qtl.pos.c2=qtl.pos2,freq.train.qtl=freq.train.qtl,freq.train.snp=freq.train.snp,freq.test.qtl=freq.test.qtl,freq.test.snp=freq.test.snp ))
}

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#FUNCTIONS for Simulation

#----------------------------------------------------------------------------------------------------------
# Create QTL effects, TBVs & Phenotypes
Create.trait<-function(Ztrain,Ztest,varTBVfixed,n.ind,n.qtl,here,fixed,var) {
  # INPUT
  # Ztrain: The QTL matrix for generation 1
  # Ztest: The QTL matrix for generation 2
  # varTBVfixed: Desired TBV variance
  # n.ind: Number of ind per generation
  # n.qtl: Number of QTLs per chromosome
  # here: heritability of trait
  # fixed: fixed effect of trait =0
  # var: Variances of the QTL effects
  # OUTPUT
  # breed.true: TBVs of generation 1
  # breed.true.new: TBVs of generation 2
  # pheno.true: Phenotypes of generation 1
  # resid.var: Residual variance
  # u: Unsclaed QTL effects
  # u.scaled: Scaled QTL effects 
  
  u.ef=array(,dim=c(2,n.qtl.per.chrom))
  u.ef[1,]=rnorm(n.qtl, sd=sqrt(var[1,]))
  u.ef[2,]=rnorm(n.qtl, sd=sqrt(var[2,]))
  
  # Unscaled TBV gen1
  geno1=Ztrain[,,,1]+Ztrain[,,,2]-3
  TBV.nonsc=numeric(length(n.ind))
  for (i in 1:n.chrom) {
    for (k in 1:n.qtl) {
      TBV.nonsc=TBV.nonsc+geno1[,i,k]*u.ef[i,k]
    }
  }
  
  # Scaled u
  uv<-as.vector(t(u.ef))
  varTBV=var(TBV.nonsc)
  sc=uv*varTBVfixed/sqrt(varTBV)
  u.ef.scaled=matrix(sc,nrow=n.chrom,ncol=n.qtl,byrow=T)
  
  # New scaled TBVs
  TBV.true=numeric(length(n.ind))
  for (i in 1:n.chrom) {
    for (k in 1:n.qtl) {
      TBV.true=TBV.true+geno1[,i,k]*u.ef.scaled[i,k]
    }
  }
  TBV.new=numeric(length(n.ind))
  geno2=Ztest[,,,1]+Ztest[,,,2]-3
  for (i in 1:n.chrom) {
    for (k in 1:n.qtl) {
      TBV.new=TBV.new+geno2[,i,k]*u.ef.scaled[i,k]
    }
  }
  
  # Phenotypes
  VE=(var(TBV.true)/here)-var(TBV.true)
  phe.true=fixed+TBV.true+rnorm(n.ind,sd=sqrt(VE))
  
  return(list(breed.true=TBV.true,breed.true.new=TBV.new,pheno.true=phe.true,resid.var=VE,u=u.ef,u.scaled=u.ef.scaled,varTBV.unscaled=varTBV))
  
}
#-----------------------------------------------------------------------------------------------------------
# Create scaled Z matrices for evaluation
Scale.Z<-function(Ztrain,Ztest,freq,n.ind,n.snps) {
  # INPUT
  # Ztrain: generation 1
  # Ztest: generation 2
  # freq: Freq for the scaling
  # n.ind: Number of ind per generation
  # n.snps: Number of SNPs per chrom
  # OUTPUT
  # Ztrain.sc: Scaled Z matrix for generation 1
  # Ztest.sc: Scaled Z matrix for generation 2
  # Ztrain.temp: Unscaled Z matrix for generation 1
  # Ztest.temp: Unscaled Z matrix for generation 2
  
  freq1<-numeric(length(2*n.snps))
  for (i in 1:n.snps) {
    freq1[i]=freq[1,i]
    freq1[i+n.snps]=freq[2,i] }
  sum2pq1=sum(2*freq1*(1-freq1))
  
  # Generation 1
  geno1=Ztrain[,,,1]+Ztrain[,,,2]-2
  Ztrain.temp<-array(,dim=c(n.ind,n.chrom*n.snps))
  for (i in 1:n.ind) {
    for (j in 1:n.snps) {	
      Ztrain.temp[i,j]<-geno1[i,1,j] 
      Ztrain.temp[i,j+n.snps]<-geno1[i,2,j]}
  }
  Ztrain.sc<-array(,dim=c(n.ind,n.chrom*n.snps))
  for (j in 1:n.ind) {
    Ztrain.sc[j,]=(Ztrain.temp[j,]-(2*freq1))/sqrt(sum2pq1)
  }
  
  # Generation 2
  geno2=Ztest[,,,1]+Ztest[,,,2]-2
  Ztest.temp<-array(,dim=c(n.ind,n.chrom*n.snps))
  for (i in 1:n.ind) {
    for (j in 1:n.snps) {	
      Ztest.temp[i,j]<-geno2[i,1,j] 
      Ztest.temp[i,j+n.snps]<-geno2[i,2,j]}
  }
  Ztest.sc<-array(,dim=c(n.ind,n.chrom*n.snps))
  for (j in 1:n.ind) {
    Ztest.sc[j,]=(Ztest.temp[j,]-(2*freq1))/sqrt(sum2pq1)
  }
  
  return(list(Ztrain.sc=Ztrain.sc,Ztest.sc=Ztest.sc,Ztrain.temp=Ztrain.temp,Ztest.temp=Ztest.temp))
}
#--------------------------------------------------------------------------------------------------------------
# Construct D categorical W10, W20, W40 simple
D.list.simple.cat<-function(n.snps,step,qtlpos1,qtlpos2,snppos1,snppos2) {
  
  D.list = list()
  for(j in 1:length(step)){
    D1 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    D2 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    D <- numeric(2*n.snps)
    
    # Chromosome 1
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos1 %in% snppos1[wL]:snppos1[wU])){
        D1[wL:wU] <- 1 }
    }
    # Chromosome 2
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos2 %in% snppos2[wL]:snppos2[wU])){
        D2[wL:wU] <- 1 }
    }
    
    D[1:n.snps] <- D1
    D[(1+n.snps):(2*n.snps)] <- D2
    D.list[[j]] <- D
    
  }
  return(list(D.list=D.list))
}
#-------------------------------------------------------------------------------------------------------------
# Construct D continuous with LD
D.list.LD.con<-function(n.ind,n.snps,n.qtl,Zsnp,Zqtl,freq1,freq2) {
  D.list=list()
  # Compute LD between all markers and the QTL on same chromosome. 
  # Chromosome 1
  D1<-numeric(n.snps)
  for (i in 1:n.snps) {
    for (j in 1:n.qtl) {
      f11=0
      f22=0
      f12=0
      f21=0
      D=0
      r2=0
      for (k in 1:n.ind) {
        for (l in 1:2) {
          if (Zsnp[k,1,i,l]==1 && Zqtl[k,1,j,l]==1) {
            f11=f11+1 }
          if (Zsnp[k,1,i,l]==2 && Zqtl[k,1,j,l]==2) {
            f22=f22+1 }
          if (Zsnp[k,1,i,l]==1 && Zqtl[k,1,j,l]==2) {
            f12=f12+1 }
          if (Zsnp[k,1,i,l]==2 && Zqtl[k,1,j,l]==1) {
            f21=f21+1 }
        }
      }
      p11=f11/(2*n.ind)
      p22=f22/(2*n.ind)
      p12=f12/(2*n.ind)
      p21=f21/(2*n.ind)
      D=(p11*p22)-(p12*p21)	
      r2=(D**2)/(freq1[1,i]*(1-freq1[1,i])*freq2[1,j]*(1-freq2[1,j]))	
      D1[i]=D1[i]+r2
    }	
  }
  # Chromosome 2
  D2<-numeric(n.snps)
  for (i in 1:n.snps) {
    for (j in 1:n.qtl) {
      f11=0
      f22=0
      f12=0
      f21=0
      D=0
      r2=0
      for (k in 1:n.ind) {
        for (l in 1:2) {
          if (Zsnp[k,2,i,l]==1 && Zqtl[k,2,j,l]==1) {
            f11=f11+1 }
          if (Zsnp[k,2,i,l]==2 && Zqtl[k,2,j,l]==2) {
            f22=f22+1 }
          if (Zsnp[k,2,i,l]==1 && Zqtl[k,2,j,l]==2) {
            f12=f12+1 }
          if (Zsnp[k,2,i,l]==2 && Zqtl[k,2,j,l]==1) {
            f21=f21+1 }
        }
      }
      p11=f11/(2*n.ind)
      p22=f22/(2*n.ind)
      p12=f12/(2*n.ind)
      p21=f21/(2*n.ind)
      D=(p11*p22)-(p12*p21)	
      r2=(D**2)/(freq1[2,i]*(1-freq1[2,i])*freq2[2,j]*(1-freq2[2,j]))	
      D2[i]=D2[i]+r2
    }	
  }
  DA <- numeric(2*n.snps)
  DA[1:n.snps] <- D1
  DA[(1+n.snps):(2*n.snps)] <- D2
  D.list[[1]]=DA
  return(list(D.list=D.list))
}
#-------------------------------------------------------------------------------------------------------------
# Construct D categorical with LD
D.list.LD.cat<-function(n.ind,n.snps,step,Zsnp,Zqtl,qtlpos1,qtlpos2,snppos1,snppos2,freq1,freq2) {
  
  D.list = list()
  for(j in 1:length(step)){
    D1 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    D2 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    
    # Chromosome 1
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos1 %in% snppos1[wL]:snppos1[wU])){
        qtl.loc.in.w <- which(qtlpos1 %in% snppos1[wL]:snppos1[wU]) # Which qtl is covered
        # Compute the LD between each marker and each QTL in the window 
        for (i in wL:wU) {
          for (m in 1:length(qtl.loc.in.w)) {
            f11=0
            f22=0
            f12=0
            f21=0
            D=0
            r2=0
            for (k in 1:n.ind) {
              for (l in 1:2) {
                if (Zsnp[k,1,i,l]==1 && Zqtl[k,1,qtl.loc.in.w[m],l]==1) {
                  f11=f11+1 }
                if (Zsnp[k,1,i,l]==2 && Zqtl[k,1,qtl.loc.in.w[m],l]==2) {
                  f22=f22+1 }
                if (Zsnp[k,1,i,l]==1 && Zqtl[k,1,qtl.loc.in.w[m],l]==2) {
                  f12=f12+1 }
                if (Zsnp[k,1,i,l]==2 && Zqtl[k,1,qtl.loc.in.w[m],l]==1) {
                  f21=f21+1 }
              }
            }
            p11=f11/(2*n.ind)
            p22=f22/(2*n.ind)
            p12=f12/(2*n.ind)
            p21=f21/(2*n.ind)
            D=(p11*p22)-(p12*p21)	
            r2=(D**2)/(freq1[1,i]*(1-freq1[1,i])*freq2[1,qtl.loc.in.w[m]]*(1-freq2[1,qtl.loc.in.w[m]]))	
            D1[i]=D1[i]+r2
          }	
        }
      } # close if
      else { D1[wL:wU]=0 }
    } # close windows
    
    # Chromosome 2
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos2 %in% snppos2[wL]:snppos2[wU])){
        qtl.loc.in.w <- which(qtlpos2 %in% snppos2[wL]:snppos2[wU]) # Which qtl is covered
        # Compute the LD between each marker and each QTL in the window 
        for (i in wL:wU) {
          for (m in 1:length(qtl.loc.in.w)) {
            f11=0
            f22=0
            f12=0
            f21=0
            D=0
            r2=0
            for (k in 1:n.ind) {
              for (l in 1:2) {
                if (Zsnp[k,2,i,l]==1 && Zqtl[k,2,qtl.loc.in.w[m],l]==1) {
                  f11=f11+1 }
                if (Zsnp[k,2,i,l]==2 && Zqtl[k,2,qtl.loc.in.w[m],l]==2) {
                  f22=f22+1 }
                if (Zsnp[k,2,i,l]==1 && Zqtl[k,2,qtl.loc.in.w[m],l]==2) {
                  f12=f12+1 }
                if (Zsnp[k,2,i,l]==2 && Zqtl[k,2,qtl.loc.in.w[m],l]==1) {
                  f21=f21+1 }
              }
            }
            p11=f11/(2*n.ind)
            p22=f22/(2*n.ind)
            p12=f12/(2*n.ind)
            p21=f21/(2*n.ind)
            D=(p11*p22)-(p12*p21)	
            r2=(D**2)/(freq1[2,i]*(1-freq1[2,i])*freq2[2,qtl.loc.in.w[m]]*(1-freq2[2,qtl.loc.in.w[m]]))	
            D2[i]=D2[i]+r2
          }	
        }
      } # close if
      else { D2[wL:wU]=0 }
    } # close windows
    
    DA <- numeric(2*n.snps)
    DA[1:n.snps] <- D1
    DA[(1+n.snps):(2*n.snps)] <- D2
    D.list[[j]]=DA
  } # close step
  return(list(D.list=D.list))
}
#-------------------------------------------------------------------------------------------------------------
# Construct D categorical for Scenario 1 and 2 (2 variances)
D.list.2var.cat<-function(n.snps,step,qtlpos1,qtlpos2,snppos1,snppos2,varia) {
  
  D.list = list()
  for(j in 1:length(step)){
    D1 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    D2 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    V1 <- numeric(n.snps) # Store type of QTL
    V2 <- numeric(n.snps) # Store type of QTL
    D <- numeric(2*n.snps)
    V <- numeric(2*n.snps)
    
    # Chromosome 1
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos1 %in% snppos1[wL]:snppos1[wU])){
        D1[wL:wU] <- 1
        # Check the variance of the qtl
        which.qtl.in.w <- which(qtlpos1 %in% snppos1[wL]:snppos1[wU]) # Which qtl were covered by the window
        if(length(which.qtl.in.w) == 1 ){ 
          if(varia[1,which.qtl.in.w] == exp(beta.upper) ){
            V1[wL:wU] <- 1 
            D1[wL:wU] <- 0 } # If only one qtl then assign 1 if high varicane
        }else{ if(length(unique(varia[1,which.qtl.in.w]))>1){ # If there were more than two and they are different, assign 1
          V1[wL:wU] <- 1 
          D1[wL:wU] <- 0
        } else {if(varia[1,which.qtl.in.w[1]] == exp(beta.upper) ){
          V1[wL:wU] <- 1 
          D1[wL:wU] <- 0 }
        }
        } # If all are same, only check first one 
      } 
    }
    # Chromosome 2
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos2 %in% snppos2[wL]:snppos2[wU])){
        D2[wL:wU] <- 1
        # Check the variance of the qtl
        which.qtl.in.w <- which(qtlpos2 %in% snppos2[wL]:snppos2[wU]) # Which qtl were covered by the window
        if(length(which.qtl.in.w) == 1 ){ 
          if(varia[2,which.qtl.in.w] == exp(beta.upper) ){
            V2[wL:wU] <- 1 
            D2[wL:wU] <- 0 } # If only one qtl then assign 1 if high varicane
        }else{ if(length(unique(varia[2,which.qtl.in.w]))>1){ # If there were more than two and they are different, assign 1
          V2[wL:wU] <- 1 
          D2[wL:wU] <- 0
        } else {if(varia[2,which.qtl.in.w[1]] == exp(beta.upper) ){
          V2[wL:wU] <- 1 
          D2[wL:wU] <- 0 }
        }
        } # If all are same, only check first one 
      } 
    }
    D[1:n.snps] <- D1
    D[(1+n.snps):(2*n.snps)] <- D2
    V[1:n.snps] <- V1
    V[(1+n.snps):(2*n.snps)] <- V2
    
    D.list[[j]] <- cbind(D,V)
  }
  return(list(D.list=D.list))
}
#-----------------------------------------------------------------------------------------------------------
# Construct D continuous with Distance
D.list.Dist.con<-function(n.snps,snppos1,snppos2,n.loci) {
  D.list=list()
  D1 <- numeric(n.snps) # Store info on marker near qtl for chrom1
  D2 <- numeric(n.snps) # Store info on marker near qtl for chrom2
  
  for (i in 1:n.snps) {
    D1[i]=min(abs(snppos1[i]-snppos1[1]),abs(snppos1[i]-snppos1[n.snps]))/(n.loci/2)
    D2[i]=min(abs(snppos2[i]-snppos2[1]),abs(snppos2[i]-snppos2[n.snps]))/(n.loci/2)
  }
  DA <- numeric(2*n.snps)
  DA[1:n.snps] <- D1
  DA[(1+n.snps):(2*n.snps)] <- D2
  D.list[[1]]=DA
  return(list(D.list=D.list))
}
#-----------------------------------------------------------------------------------------------------------
# Construct D categorical with Distance
D.list.Dist.cat<-function(n.snps,step,qtlpos1,qtlpos2,snppos1,snppos2,n.loci) {
  
  D.list = list()
  for(j in 1:length(step)){
    D1 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    D2 <- numeric(n.snps) # Store info on marker near qtl for chrom1
    
    # Chromosome 1
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos1 %in% snppos1[wL]:snppos1[wU])){
        for (i in wL:wU) {
          D1[i]=min(abs(snppos1[i]-snppos1[1]),abs(snppos1[i]-snppos1[n.snps]))/(n.loci/2)
        }
      } # close if
      else { D1[wL:wU]=0 }
    } # close windows
    
    # Chromosome 2
    nr.windows <- n.snps/step[j]
    for(s in 1:nr.windows){
      # Find the boundaries for the current window
      wL <- 1 + (s-1)*step[j]
      wU <- s*step[j]
      # Check if the window covers any qtl
      if( any(qtlpos2 %in% snppos2[wL]:snppos2[wU])){
        for (i in wL:wU) {
          D2[i]=min(abs(snppos2[i]-snppos2[1]),abs(snppos2[i]-snppos2[n.snps]))/(n.loci/2)
        }
      } # close if
      else { D2[wL:wU]=0 }
    } # close windows
    
    DA <- numeric(2*n.snps)
    DA[1:n.snps] <- D1
    DA[(1+n.snps):(2*n.snps)] <- D2
    D.list[[j]]=DA
  } # close step
  return(list(D.list=D.list))
}
